==> scribe-0.1.0-20180305.185151-54.jar: chronos/java8_time.clj <==
(ns chronos.java8-time
  (:require [chronos.specs :as cs]
            [chronos.utils :as u])
  (:import [java.time Instant LocalDate LocalTime]))

;;,--------
;;| Instant
;;`--------
(defmethod cs/instant? java.time.Instant [_] true)

(defmethod cs/to-epoch-millis java.time.Instant
  [^Instant i]
  (.toEpochMilli i))

(defmethod cs/of-epoch-millis java.time.Instant
  [_ ms]
  (Instant/ofEpochMilli ms))

(defmethod cs/to-epoch-micros java.time.Instant
  [^Instant i]
  ;; TODO ??? something like this?
  #_(+ (secons->micros (.getLong i))
       (nanos->micros! (.getNano i)))
  (u/not-implemented!))

(defmethod cs/of-epoch-micros java.time.Instant
  [_ µs]
  (u/not-implemented!))

;;,----------
;;| LocalDate
;;`----------
(defmethod cs/local-date? java.time.LocalDate [_] true)

(defmethod cs/to-epoch-days java.time.LocalDate
  [^LocalDate ld]
  (.toEpochDay ld))

(defmethod cs/of-epoch-days java.time.LocalDate
  [_ days]
  (LocalDate/ofEpochDay days))

;;,----------
;;| LocalTime
;;`----------
(defmethod cs/local-time? java.time.LocalTime [_] true)

(defmethod cs/to-millis-of-day java.time.LocalTime
  [^LocalTime lt]
  (-> lt .toNanoOfDay u/nanos->millis!))

(defmethod cs/of-millis-of-day java.time.LocalTime
  [_ ms]
  (LocalTime/ofNanoOfDay (u/millis->nanos ms)))

(defmethod cs/to-micros-of-day java.time.LocalTime
  [^LocalTime lt]
  (-> lt .toNanoOfDay u/nanos->micros!))

(defmethod cs/of-micros-of-day java.time.LocalTime
  [_ µs]
  (LocalTime/ofNanoOfDay (u/micros->nanos µs)))

;;,-------
;;| Period
;;`-------

;; ??? java.time.Period doesn't match Avro period?

==> scribe-0.1.0-20180306.194204-59.jar: chronos/java8_time.clj <==
(ns chronos.java8-time
  (:require [chronos.specs :as cs]
            [chronos.utils :as u])
  (:import [java.time Instant LocalDate LocalTime]))

;;,--------
;;| Instant
;;`--------
(defmethod cs/instant? java.time.Instant [_] true)

(defmethod cs/to-epoch-milli java.time.Instant
  [^Instant i]
  (.toEpochMilli i))

(defmethod cs/of-epoch-milli java.time.Instant
  [_ ms]
  (Instant/ofEpochMilli ms))

(defmethod cs/to-epoch-micro java.time.Instant
  [^Instant i]
  ;; TODO ??? something like this?
  #_(+ (secons->micros (.getLong i))
       (nanos->micros! (.getNano i)))
  (u/not-implemented!))

(defmethod cs/of-epoch-micro java.time.Instant
  [_ µs]
  (u/not-implemented!))

;;,----------
;;| LocalDate
;;`----------
(defmethod cs/local-date? java.time.LocalDate [_] true)

(defmethod cs/to-epoch-day java.time.LocalDate
  [^LocalDate ld]
  (.toEpochDay ld))

(defmethod cs/of-epoch-day java.time.LocalDate
  [_ days]
  (LocalDate/ofEpochDay days))

;;,----------
;;| LocalTime
;;`----------
(defmethod cs/local-time? java.time.LocalTime [_] true)

(defmethod cs/to-milli-of-day java.time.LocalTime
  [^LocalTime lt]
  (-> lt .toNanoOfDay u/nanos->millis!))

(defmethod cs/of-milli-of-day java.time.LocalTime
  [_ ms]
  (LocalTime/ofNanoOfDay (u/millis->nanos ms)))

(defmethod cs/to-micro-of-day java.time.LocalTime
  [^LocalTime lt]
  (-> lt .toNanoOfDay u/nanos->micros!))

(defmethod cs/of-micro-of-day java.time.LocalTime
  [_ µs]
  (LocalTime/ofNanoOfDay (u/micros->nanos µs)))

;;,-------
;;| Period
;;`-------

;; ??? java.time.Period doesn't match Avro period?
